/*
  BCK.CPP Brain Construction Kit
  written by Aquiles Luna-Rodrguez, Kalckreuthweg 89-Zi63, 22607 Hamburg
  tel. (Germany) 040 892670    Internet: pz4a004@rrz.uni-hamburg.de
  compiles with Turbo C++ 3.0 for DOS
  memory model HUGE, test stack overflow, signed chars.
*/
         //cells variables are partly stored in the stack by main()
extern unsigned _stklen = 55000u; //55000u N_C 100 MAX_SYN 36 MNT 8

#include <stdio.h>
#include <conio.h>    //getch gotoxy kbhit
#include <string.h>   //strcpy strcmp
#include <math.h>     //pow abs log
#include <graphics.h> //initgraph putpixel getpixel getmaxy setcolor line
#include <dos.h>      //delay

//global constants
const int
   //general caracteristics of cellular automata
   N_C      = 100, //total Number of Cells
   MAX_SYN  =  36, //MAXimum number of SYNapses by cell, must be < N_C
   MNT      =   8, //Maximum number of NeuroTransmiter types
   R_P_SPAN =  15, //"normal" span of Refractory Period
   //other values used in compile time
   WO_SZ_X  =  20, //size of world, including two-row margins (in squares)
   WO_SZ_Y  =  20,
   WS       =   8, //side-size in pixels of World's squares
   WO_PO_X  = 504, //position of world map, upper-left corner (in pixels)
   WO_PO_Y  = 194, //..also vertical margin for plot-depo()
   WAVES    =   4, //how many "EEG" of groups of cells are to be ploted

   //distribution of cells by zone
   RET  =  0,  //RETina (begins with 0, 8 pixels * 3 colors == 24 cells)
   NOR  = 24,  //orientation NORth (3 cells by direction)
   EAS  = 27,  //orientation EASt
   SOU  = 30,  //orientation SOUth
   WES  = 33,  //orientation WESt
   LEF  = 36,  //motor zone LEFt (6 cells by behavior)
   MOV  = 42,  //motor zone MOVement
   RIG  = 48,  //motor zone RIGht
   FREE = 54;  //no special input or output beginning at FREE
const float  FT = 100.01; //FormaT to print 1.0 as 100

//global parameters, user-defined
//these are only dummy values, definition is in params.bck and struc.bck
int    EegB[WAVES] ={  0, 0, 0, 0},
       EegE[WAVES] ={ -1,-1,-1,-1},
       PLEAb =  0, //pleasure and pain centers are independent..
       PLEAe = -1, //..from cell type
       PAINb =  0,
       PAINe = -1,
       HL_PRI [ MNT ],//Half-Life  of PRImary neurotransmitter (generations)
       HL_TRA [ MNT ],//TRAnsformation factor pri->sec (sec += HL_TRA * pri)
       HL_SEC [ MNT ],//Half-Life  of SECondary neurotransmitter
       HL_AUX = 0;    //Half-Life of AUXiliary input
       //NeuroTransmitter Half-Life (as multiplication factor)
double HL_P [ MNT ],   //Primary
       HL_T [ MNT ],   //Transformation
       HL_S [ MNT ],   //Secondary
       HL_A = 0.0,     //Auxiliary input
       NT_TH  [ MNT ], //NT ThresHold;
       NT_PO  [ MNT ], //NT POwer,
       FOR   [MNT][MNT];  //chemical synaptic growth formula
int    ACTIVE[MNT][MNT];  //which syn. types grow in which cell types
double CAT   [MNT][MNT],  //NT[line] "CATalizes" NT[column]
       ATP = 0.0,         //energy substance production ("ATP")
       LAC = 0.0,         //fatigue substance production ("LACtic acid")
       STI = 0.0,         //STImulus-> retina, orientation, pleasure/pain.
       MOTOR_TH = 0.0,    //MOTOR ThresHold, depends on # of motor cells
       REF_PERn[MNT][R_P_SPAN], //REFractory PERiod (NT absorption)
       REF_PERc[MNT][R_P_SPAN]; //REFractory PERiod (cell activity)

int  SCR = 0,  //SCReen type  0 == color,  1 == black & white
     SEED = 345, //random number generator's default SEED
     GREY[2][16] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15, //dummy
                    15,14,13,12,11,10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0},
     NtCol[2][MNT] = {15,15,15,15,15,15,15,15, //dummy
                      15,15,15,15,15,15,15,15};

//program's internal global variables
int  RndVec[8] = {0,SEED,SEED,SEED,SEED,SEED,SEED,SEED}; //random num. gen.
char NeiSta[N_C][N_C],          //Neighbor's Status, defined in cold-start
     World[WO_SZ_Y][WO_SZ_X];   //defined in set-World-up
double Left        = 0.0,       //amount of activity in motor zone
       Right       = 0.0,       //idem
       Move        = 0.0;       //idem
long   Gen         = 0;         //counter of automata's generations
int    Pox         = 9,         //Position, X..
       Poy         = 9,         //..& Y axis
       TEST = 0, // 1 for developement & debugging
       TimePos     = -1;        //X-axis position for depo & EEG
char   Direc       = 'n',       //direction: 'n'orth, 'e'ast, 's'outh, 'w'est
       DispUp      = 'M';       //display up (paraMs Bars For...)

/*************************** functions ******************************/

void clean_line(int which) {
   gotoxy(1,which);
   printf("                                        ");
   printf("                                       ");
   gotoxy(1,which);
}//clean_line


void clean_upside(void) {
   int c;
   for(c=2; c<12; clean_line(c++) ) ;
   gotoxy(1,2);
}//clean_upside


void menu_line(void) {
   clean_line(1);
   printf(" ");
   switch(DispUp) {
      case 'B':printf("[esc] 0 1 2 3 bars paraMs For Acti ");
               printf(" Cat ref-Per cEll Neig Wei Reset saVe");
               break;
      case 'M':printf("[esc] 0 1 2 3 Bars params For Acti ");
               printf(" Cat ref-Per cEll Neig Wei Reset saVe cHa");
               break;
      case 'F':printf("[esc] 0 1 2 3 Bars paraMs for Acti ");
               printf(" Cat ref-Per cEll Neig Wei Reset saVe cHa");
               break;
      case 'A':printf("[esc] 0 1 2 3 Bars paraMs For acti ");
               printf(" Cat ref-Per cEll Neig Wei Reset saVe cHa");
               break;
      case 'C':printf("[esc] 0 1 2 3 Bars paraMs For Acti ");
               printf(" cat ref-Per cEll Neig Wei Reset saVe cHa");
               break;
      case 'P':printf("[esc] 0 1 2 3 Bars paraMs for Acti ");
               printf(" Cat ref-per cEll Neig Wei Reset saVe cHa");
               break;
      default:printf("error: DispUp undefined");
      }
}//menu_line


double ask_double(char message[],double old) {
   double tempd = 0.0;
   char line[40] = "\0";
   clean_line(1);
   printf("%s %lf -> ",message,old);
   gets(line);
   if(line[0] =='\0') { menu_line(); return(old); }
      else {
      sscanf(line,"%lf",&tempd);
      menu_line();
      return(tempd);
      }
}//ask_double


int ask_int(char message[],int old) {
   int tempi = 0;
   char line[40] = "\0";
   clean_line(1);
   printf("%s ",message);
   if(old != -1) printf("%d  new value? ",old);
   gets(line);
   if(line[0] =='\0') { menu_line(); return(old); }
      else {
      sscanf(line,"%d",&tempi);
      menu_line();
      return(tempi);
      }
}//ask_int


int rnd(int range) { //returns integer random numbers between 0 and range-1
   long templ = 0;
   int c;
   if (RndVec[0] > 6) RndVec[0] = 1; //RndVec[0] points to the next
      else RndVec[0]++;
   for(c=1; c<8; c++)
      templ += RndVec[c];
   templ %= 4001; //modulo [prime number]
   RndVec[RndVec[0]] = (int)templ;
   return( (int) (templ * range / 4001) );
}//rnd
void reset_generator(void) {
   int c;
   for(RndVec[0] = 0, c=1; c<8; RndVec[c] = SEED, c++) ;
}//reset_generator


double half_life(int generations) {
   double hi = 1.0,
          lo = 0.0,
          tempd = 0.0;
   int c;
   if(generations < 1) return(0.0);
   for(c=0; c<20; c++) {
      tempd = (hi + lo) / 2.0;
      if (pow(tempd,generations) < 0.5) lo = tempd;
      if (pow(tempd,generations) > 0.5) hi = tempd;
      }
   return(tempd);
}//half_life


//cold_start starts the graphics,
//cleans up all global vars. not defined on declaration (except World),
//and cell parts not defined by reset()
void cold_start(int Type   [],
                int SynNum [],
                float SynEq  [][MNT],
                int   SynType[][MAX_SYN],
                int   *Syn   [][MAX_SYN]) {
   int c,c2;
   int grdriver = DETECT,grmode,error;  //VGALO
   initgraph(&grdriver,&grmode,"");
   error = graphresult();
   if(error) {printf("\n\007graphic error %d",error); getch();}
   for(c=0; c<MNT; c++) {
      HL_PRI[c] = 0; HL_P[c] = 0.0;
      HL_TRA[c] = 0; HL_T[c] = 0.0;
      HL_SEC[c] = 0; HL_S[c] = 0.0;
      NT_TH[c] = 0.0;
      NT_PO[c] = 0.0;
      for(c2=0; c2<MNT; c2++) {
         FOR   [c][c2] = 0.0;
         ACTIVE[c][c2] = 0;
         CAT   [c][c2] = 0.0;
         }
      for(c2=0; c2<R_P_SPAN; c2++) {
         REF_PERn[c][c2] = 0.0;
         REF_PERc[c][c2] = 0.0;
         }
      }
   for(c=0; c<N_C; c++) //status of neighbor, 0 no conection, 99 itself
      for(c2=0; c2<N_C; c2++)
         NeiSta[c][c2] = (c != c2) ?  0 : 99;
   for(c=0; c<N_C; c++) {
      Type[c]    =  0; //types 0 and 1 are undefined; allowed: 2->MNT
      SynNum[c]  =  0;
      for(c2 = 0; c2<MNT; SynEq[c][c2] = 1.0, c2++) ; //no equalization
      for(c2=0; c2<MAX_SYN; c2++) {
         SynType[c][c2] =    0;  //type 0 undefined
         Syn    [c][c2] = NULL;  //doesn't points to anything yet
         }
      }
}//cold_start


void set_World_up(FILE *dta_fil, int load_fil) {
static char world_model[WO_SZ_Y][WO_SZ_X];
   char line[80];
   int c,c2;
   if(load_fil)
      for(c=0; c<WO_SZ_Y; c++) {
         fscanf(dta_fil,"%s",&line);
         for(c2=0; c2<WO_SZ_X; c2++)
            world_model[c][c2] = line[c2];
         }
   reset_generator();
   for(c=0; c<WO_SZ_Y; c++)
      for(c2=0; c2<WO_SZ_X; c2++)
         switch (world_model[c][c2]) {
            case '0':
            case '#':World[c][c2] = 0; break; //red   (stone)
            case '1':
            case '.':World[c][c2] = 1; break; //green (free)
            case '2':
            case '+':World[c][c2] = 2; break; //blue  (food)
            case '3':
            case '*':World[c][c2] = 3; break; //yellow
            case '4':World[c][c2] = 4; break; //cyan
            case '5':World[c][c2] = 5; break; //magenta
            case '6':World[c][c2] = 6; break; //white
            case '7':World[c][c2] = 7; break; //black
            case 'r':World[c][c2] = rnd(3); break;
            case 's':World[c][c2] = rnd(4); break;
            case 't':World[c][c2] = rnd(5); break;
            case 'u':World[c][c2] = rnd(6); break;
            case 'v':World[c][c2] = rnd(7); break;
            case 'w':World[c][c2] = rnd(8); break;
            default :World[c][c2] = 0;
            }
}//set_World_up


void reset(int State  [],
           int LastTir[],
           float AuxInp[],
           float NtInp1[][MNT],
           float NtInp2[][MNT],
           float SynWei[][MAX_SYN],
           float SyNt1 [][MAX_SYN],
           float SyNt2 [][MAX_SYN]) {
   int c,c2;
   set_World_up(0,0); //resets generator anyway
   Left    = 0.0;
   Right   = 0.0;
   Move    = 0.0;
   Gen     = 0;
   Pox     = 9;
   Poy     = 9;
   TimePos = -1;
   Direc   = 'n';
   reset_generator();
   for(c=0; c<N_C; c++) {
      State[c]   =  rnd(R_P_SPAN - 2) + 2; //initial states are random (!0)
      LastTir[c] =  R_P_SPAN;
      AuxInp[c]  =  0.0;
      for(c2 = 0; c2<MNT; c2++) {
         NtInp1[c][c2] = 0.0;
         NtInp2[c][c2] = 0.0;
         }
      for(c2=0; c2<MAX_SYN; c2++) {
         SynWei  [c][c2] =  1.0;
         SyNt1   [c][c2] =  0.0;
         SyNt2   [c][c2] =  0.0;
         }
      }
}//reset


void read_nei(int Type  [],
              int State [],
              int SynNum[],
              int SynType[][MAX_SYN],
              int *Syn   [][MAX_SYN],
              int reads,int readed,int syntype) { //>read---
   Syn    [reads][SynNum[reads]] = &State[readed];
   SynType[reads][SynNum[reads]] = (syntype < 0) ? Type[readed] : syntype;
   NeiSta[reads][readed] = SynType[reads][SynNum[reads]];
   ++SynNum[reads];
}//read_nei


int wrap(int first, int last, int pos) {
   if (pos < first) pos += (last - first + 1);
      else if (pos > last)  pos -= (last - first + 1);
   return(pos);
}//wrap


void loc_nei(int first, int last, int syn_typ, //local neighborhood
             int n0, int n1, int n2, int n3,
             int Type  [],
             int State [],
             int SynNum[],
             int SynType[][MAX_SYN],
             int *Syn   [][MAX_SYN]) {
   const int loc_neis = 4;
   int c,c2,
       nei_pos,
       nei[loc_neis];
   nei[0] = n0; nei[1] = n1; nei[2] = n2; nei[3] = n3;
   if( (first < 0) || (last >= N_C) || (first >= last) ) {
      printf("\n\007 out of range in loc_nei ");
      printf("\nfirst %sd  last %2d  type %2d    %3d  %3d  %3d  %3d",
             first,last,syn_typ,n0,n1,n2,n3);
      getch();
      return;
      }
   for(c=0; c<loc_neis; c++)
      if( abs(nei[c]) > (last-first) ) {
         printf("\n\007 neighbor distance bigger than target size in loc_nei ");
         getch();
         return;
         }
   for(c=first; c<=last; c++)
      for(c2=0; c2<loc_neis; c2++) {
         if(!nei[c2]) continue; //neighbor position == 0 -> no synapse
         if(SynNum[c] + 1 >= MAX_SYN) {
            printf("\n\007 too many neighbors on cell %2d",c);
            getch();
            continue;}
         nei_pos = wrap(first, last, c+nei[c2]);
         if(NeiSta[c][nei_pos] != 0) {
            printf("\n  occupied cell %d   neighbor %d (press a key)"
                   ,c,nei_pos);
            getch();
            continue;}
         Syn    [c][SynNum[c]] = &State[nei_pos];
         SynType[c][SynNum[c]] = (syn_typ < 0) ? Type[nei_pos] : syn_typ;
         NeiSta[c][nei_pos] = SynType[c][SynNum[c]];
         ++SynNum[c];
         }
}//loc_nei

           //zone begin,id end ,quantity of syn,id type,neighbor's address
void rnd_nei(int z_b, int z_e, int s_q, int s_t, int n_b, int n_e,
             int Type  [],
             int State [],
             int SynNum[],
             int SynType[][MAX_SYN],
             int *Syn   [][MAX_SYN]) {
   int c,c2,
       target_size,
       tempi;
   target_size = n_e - n_b + 1;
   for(c=z_b; c <= z_e; c++) {
      if(SynNum[c]+s_q > MAX_SYN) {
         printf("\n\007too many synapses in cell:%d (%d)",c,SynNum[c]);
         printf("  %d %d %d %d %d %d",z_b,z_e,s_q,s_t,n_b,n_e);
         getch();
         goto jump_cell;
         }
      for(c2=SynNum[c]; c2<SynNum[c]+s_q; c2++) {
         do tempi = rnd(target_size);
            while (NeiSta[c][n_b + tempi]);
         Syn[c][c2] = &State [n_b + tempi];
         //if s_t < 0 take the Type of neighbor
         SynType[c][c2] = (s_t < 0) ? Type[n_b + tempi] : s_t;
         NeiSta[c][n_b + tempi] = SynType[c][c2];
         }
      SynNum[c] += s_q;
      jump_cell:
      }
}//rnd_nei


int params_file(int load) {
   const int code_size = 15,
             maxcodes = 17,
             big = 256;
   const char *in_name   = "param01.bck\0",     //default input file
              *out_name  = "param01.bck\0",     //default output file
              *test_name = "0new\0";            //wild experiments
   FILE *in_fil, *out_fil;
   char code_list[maxcodes][code_size] = { ">HL_PRI-------\0", //0
                                           ">HL_TRA-------\0",
                                           ">HL_SEC-------\0",
                                           ">HL_AUX-------\0",
                                           ">NT_TH--------\0",
                                           ">NT_PO--------\0", //5
                                           ">FOR----------\0",
                                           ">ACTIVE-------\0",
                                           ">CAT----------\0",
                                           ">ATP----------\0",
                                           ">LAC----------\0", //10
                                           ">STI----------\0",
                                           ">MOTOR_TH-----\0",
                                           ">REF_PERn-----\0",
                                           ">REF_PERc-----\0",
                                           ">stop---------\0", //15
                                           ">end.---------\0" };
   char code[code_size] = "\0",
        big_line[big] = "\0",
        dumch;
   int c,c2,c3;
   gotoxy(1,1);
   while(load) {
      if(TEST) strcpy(big_line,test_name); else strcpy(big_line,in_name);
      printf("\n\n[RETURN]->load %s   [q]->quit ",big_line);
      printf("  [any letter]->type a file name ");
      if (TEST) dumch = 13; else dumch = getch();
      if(dumch != 13)
         if((dumch == 'q')||(dumch =='Q')) return(1);
            else {
               printf("\n\nplease type [file name] [RETURN]  ");
               scanf("%s",&big_line);
               }
      if( (in_fil = fopen(big_line,"r+")) == NULL)
         printf("\n\ncan't find data file \"%s\"",big_line);
         else break;
      }
   while( (load) && (!feof(in_fil)) ) {
      fgets(code,code_size,in_fil);
      if(code[0] != '>') continue;  //ignore lines not beginning with ">"
      for(c=0; c<maxcodes; c++)
         if( strcmp(code,code_list[c]) == 0 ) {
            printf("\n%s",code);         //print heading
            fgets(big_line,big,in_fil); //read rest of the line
            switch(c) {
               case  0:for(c2=0; c2<MNT;
                          fscanf(in_fil,"%d",&HL_PRI[c2]),
                          printf(" %4d ",HL_PRI[c2]), c2++) ;
                       break;
               case  1:for(c2=0; c2<MNT;
                          fscanf(in_fil,"%d",&HL_TRA[c2]),
                          printf(" %4d ",HL_TRA[c2]), c2++) ;
                       break;
               case  2:for(c2=0; c2<MNT;
                          fscanf(in_fil,"%d",&HL_SEC[c2]),
                          printf(" %4d ",HL_SEC[c2]), c2++) ;
                       break;
               case  3:fscanf(in_fil,"%d",&HL_AUX);
                       HL_A = half_life(HL_AUX);
                       printf(" %4d",HL_AUX);
                       break;
               case  4:for(c2=0; c2<MNT;
                          fscanf(in_fil,"%lf",&NT_TH[c2]),
                          printf(" %4d ",(int)(NT_TH[c2] * FT) ), c2++) ;
                       break;
               case  5:for(c2=0; c2<MNT;
                          fscanf(in_fil,"%lf",&NT_PO[c2]),
                          printf(" %4d ",(int)(NT_PO[c2] * FT) ), c2++) ;
                       break;
               case  6:for(c2=0; c2<MNT; c2++) {
                          printf("\n         ");
                          for(c3=0; c3<MNT; c3++) {
                             fscanf(in_fil,"%lf",&FOR[c2][c3]);
                             printf(" %1.2lf ",FOR[c2][c3]);
                             }
                          }
                       break;
               case  7:for(c2=0; c2<MNT; c2++) {
                          printf("\n         ");
                          for(c3=0; c3<MNT; c3++) {
                             fscanf(in_fil,"%d",&ACTIVE[c2][c3]);
                             printf(" %4d ",ACTIVE[c2][c3]);
                             }
                          }
                       break;
               case  8:for(c2=0; c2<MNT; c2++) {
                          printf("\n         ");
                          for(c3=0; c3<MNT; c3++) {
                             fscanf(in_fil,"%lf",&CAT[c2][c3]);
                             printf(" %1.2lf ",CAT[c2][c3]);
                             }
                          }
                       break;
               case  9:fscanf(in_fil,"%lf",&ATP);
                       printf(" %4d ",(int)(ATP * FT) );
                       break;
               case 10:fscanf(in_fil,"%lf",&LAC);
                       printf(" %4d ",(int)(LAC * FT) );
                       break;
               case 11:fscanf(in_fil,"%lf",&STI);
                       printf(" %4d ",(int)(STI * FT) );
                       break;
               case 12:fscanf(in_fil,"%lf",&MOTOR_TH);
                       printf(" %4d ",(int)(MOTOR_TH * FT) );
                       break;
               case 13:for(c2=0; c2<MNT; c2++) {
                          printf("\n");
                          for(c3=0; c3<R_P_SPAN; c3++) {
                             fscanf(in_fil,"%lf",&REF_PERn[c2][c3]);
                             printf(" %3d",(int)(REF_PERn[c2][c3] * 100.01) );
                             }
                          }
                       break;
               case 14:for(c2=0; c2<MNT; c2++) {
                          printf("\n");
                          for(c3=0; c3<R_P_SPAN; c3++) {
                             fscanf(in_fil,"%lf",&REF_PERc[c2][c3]);
                             printf(" %3d",(int)(REF_PERc[c2][c3] * 100.01) );
                             }
                          }
                       break;
               case 15:printf("    press any key");
                       getch();
                       break;
               case 16:printf("\n  END OF DATA press any key");
                       fclose(in_fil);
                       if(!TEST) getch();
                       for(c=1; c<32; clean_line(c), c++) ;
                       return(0);
               }
            }
      }
   while(!load) { //***************saving****************
      strcpy(big_line,out_name);
      clean_line(1);
      printf("[RETURN]->save as %s   [q]->quit ",big_line);
      printf("  [any letter]->type a file name ");
      dumch = getch();
      if(dumch != 13)  //if not a [RETURN]
         if((dumch == 'q')||(dumch == 'Q')) return(1);
            else {
               clean_line(1);
               printf("please type [file name] [RETURN]  ");
               scanf("%s",&big_line);
               }
      if( (out_fil = fopen(big_line,"w+")) == NULL) {
         clean_line(1);
         printf("can't save file \"%s\"",big_line);
         delay(2000);
         }
         else break;
      }
   if(!load) for(c=0; c<maxcodes-2; c++){   //maxcodes-2: skip stop & end
      fprintf(out_fil,"\n%s\n",code_list[c]);  //print heading
      switch(c) {
         case 0:for(c2=0; c2<MNT; c2++)
                   fprintf(out_fil," %4d ",HL_PRI[c2]);
                break;
         case 1:for(c2=0; c2<MNT; c2++)
                   fprintf(out_fil," %4d ",HL_TRA[c2]);
                break;
         case 2:for(c2=0; c2<MNT; c2++)
                   fprintf(out_fil," %4d ",HL_SEC[c2]);
                break;
         case 3:fprintf(out_fil," %4d",HL_AUX); break;
         case 4:for(c2=0; c2<MNT; c2++)
                   fprintf(out_fil," %1.4lf ",NT_TH[c2]);
                break;
         case 5:for(c2=0; c2<MNT; c2++)
                   fprintf(out_fil," %1.4lf ",NT_PO[c2]);
                break;
         case  6:for(c2=0; c2<MNT; c2++) {
                    for(c3=0; c3<MNT; c3++)
                       fprintf(out_fil," %1.4lf ",FOR[c2][c3]);
                    fprintf(out_fil,"\n");
                    }
                 break;
         case  7:for(c2=0; c2<MNT; c2++) {
                    for(c3=0; c3<MNT; c3++)
                       fprintf(out_fil," %4d ",ACTIVE[c2][c3]);
                    fprintf(out_fil,"\n");
                    }
                 break;
         case  8:for(c2=0; c2<MNT; c2++) {
                    for(c3=0; c3<MNT; c3++)
                        fprintf(out_fil," %1.4lf ",CAT[c2][c3]);
                    fprintf(out_fil,"\n");
                    }
                 break;
         case 9:fprintf(out_fil," %1.4lf",ATP); break;
         case 10:fprintf(out_fil," %1.4lf",LAC); break;
         case 11:fprintf(out_fil," %1.4lf",STI); break;
         case 12:fprintf(out_fil," %1.4lf",MOTOR_TH); break;
         case 13:for(c2=0; c2<MNT; c2++) {
                    for(c3=0; c3<R_P_SPAN; c3++)
                       fprintf(out_fil," %1.2lf",REF_PERn[c2][c3]);
                    fprintf(out_fil,"\n");
                    }
                 break;
         case 14:for(c2=0; c2<MNT; c2++) {
                    for(c3=0; c3<R_P_SPAN; c3++)
                       fprintf(out_fil," %1.2lf",REF_PERc[c2][c3]);
                    fprintf(out_fil,"\n");
                    }
                 break;
         }
      }
   if(!load) {
      fprintf(out_fil,"%s\n",code_list[maxcodes-1]); //print end.---
      fclose(out_fil);
      menu_line();
      }
   return(0);
}//params_file


int struc_file(int Type  [],
                   int State [],
                   int SynNum[],
                   int SynType[][MAX_SYN],
                   int *Syn   [][MAX_SYN]) {
   const int code_size =  15, //0..13, 14 == '\0'
             pas_sz    = 256, //pascal size of strings
             max_code  =  15; //total of recogniced codes
   const char *def_fil = "struc01.bck\0";
   FILE *dta_fil;
   char big_line[pas_sz] = "\0";
   char code [code_size] = "\0";
   char code_list[max_code][code_size] = { ">Eeg0---------\0", //0
                                           ">Eeg1---------\0",
                                           ">Eeg2---------\0",
                                           ">Eeg3---------\0",
                                           ">PLEA---------\0",
                                           ">PAIN---------\0", //5
                                           ">GREY---------\0",
                                           ">NtCol--------\0",
                                           ">World--------\0",
                                           ">Type---------\0",
                                           ">read---------\0", //10
                                           ">loc_nei------\0",
                                           ">rnd_nei------\0",
                                           ">stop---------\0",
                                           ">end.---------\0" };
   int c,c2,c3,
       dum1,dum2,dum3,dum4,dum5,dum6,dum7; //dummy integer 1, 2, ...??
   char dumch;
   do {
      printf("\n\n[RETURN]->load %s   [q]->quit",def_fil);
      printf("  [any letter]->type a file name ");
      if(TEST) dumch = 13; else dumch = getch();
      if(dumch == 13) strcpy(big_line,def_fil);
         else if(dumch == 'q') return(1);
            else {
               printf("\n\nplease type [file name] [RETURN]  ");
               scanf("%s",&big_line);
               }
      if( (dta_fil = fopen(big_line,"r+")) == NULL)
         printf("\n\ncan't find data file \"%s\"",big_line);
         else break;
      }while(1);
   do {
      fgets(code,code_size,dta_fil);
      if(code[0] != '>') continue;
      for(c=0; c<max_code; c++)
         if( strcmp(code,code_list[c]) == 0 ) {
            printf("\n%s",code);
            switch(c) {
               case  0:fscanf(dta_fil,"%d%d",&EegB[0],&EegE[0]);
                       printf(" %3d %3d",EegB[0],EegE[0]); break;
               case  1:fscanf(dta_fil,"%d%d",&EegB[1],&EegE[1]);
                       printf(" %3d %3d",EegB[1],EegE[1]); break;
               case  2:fscanf(dta_fil,"%d%d",&EegB[2],&EegE[2]);
                       printf(" %3d %3d",EegB[2],EegE[2]); break;
               case  3:fscanf(dta_fil,"%d%d",&EegB[3],&EegE[3]);
                       printf(" %3d %3d",EegB[3],EegE[3]); break;
               case  4:fscanf(dta_fil,"%d%d",&PLEAb,&PLEAe);
                       printf(" %3d %3d",PLEAb,PLEAe); break;
               case  5:fscanf(dta_fil,"%d%d",&PAINb,&PAINe);
                       printf(" %3d %3d",PAINb,PAINe); break;
               case  6:fgets(big_line,pas_sz,dta_fil);
                       for(c2=0; c2<2; c2++) {
                          printf("\n");
                          for(c3=0; c3<16; c3++) {
                             fscanf(dta_fil,"%d",&GREY[c2][c3]);
                             printf("%4d",GREY[c2][c3]);
                             }
                          }
                        break;
               case  7:fgets(big_line,pas_sz,dta_fil);
                       for(c2=0; c2<2; c2++) {
                          printf("\n");
                          for(c3=0; c3<MNT; c3++) {
                             fscanf(dta_fil,"%d",&NtCol[c2][c3]);
                             printf("%4d",NtCol[c2][c3]);
                             }
                          }
                        break;
               case  8:set_World_up(dta_fil,1); break;
               case  9:fscanf(dta_fil,"%d%d%d",&dum1,&dum2,&dum3);
                       printf(" %3d %3d %3d ",dum1,dum2,dum3);
                       if( (dum3 < 2) || (dum3 >= MNT) ) {
                          printf("\n\007wrong cell type: %d\n",dum3);
                          getch();
                          break;
                          }
                       for(c2=dum1; c2 <= dum2; c2++)
                          Type[c2] = dum3;
                       break;
               case 10:fscanf(dta_fil,"%d%d%d",&dum1,&dum2,&dum3);
                       printf(" %3d %3d %3d",dum1,dum2,dum3);
                       read_nei(Type,State,SynNum,SynType,Syn,
                                dum1,dum2,dum3);
                       break;
               case 11:fscanf(dta_fil,"%d%d%d%d%d%d%d",
                          &dum1,&dum2,&dum3,&dum4,&dum5,&dum6,&dum7);
                       printf(" %3d %3d %3d %3d %3d %3d %3d",
                          dum1,dum2,dum3,dum4,dum5,dum6,dum7);
                       loc_nei(dum1,dum2,dum3,dum4,dum5,dum6,dum7,
                          Type,State,SynNum,SynType,Syn);
                       break;
               case 12:fscanf(dta_fil,"%d%d%d%d%d%d",
                          &dum1,&dum2,&dum3,&dum4,&dum5,&dum6);
                       printf(" %3d %3d %3d %3d %3d %3d",
                          dum1,dum2,dum3,dum4,dum5,dum6);
                       rnd_nei(dum1,dum2,dum3,dum4,dum5,dum6,
                          Type,State,SynNum,SynType,Syn);
                       break;
               case 13:printf("    press any key");
                       getch();
                       break;
               case 14:printf("\n  END OF DATA press any key");
                       if(!TEST) getch();
                       goto terminate;
               }
         }
      }while(1);
   terminate:
   fclose(dta_fil);
   for(c=1; c<32; clean_line(c), c++) ;
   return(0);
}//struc_file


void hl_factors(void) {
   int c;
   for(c=0; c<MNT; c++) {
      HL_P [c] = half_life(HL_PRI[c]);
      HL_T [c] = 1.0 - half_life(HL_TRA[c]);
      HL_S [c] = half_life(HL_SEC[c]);
      }
}//hl_factors


void setEqu(int SynNum[],
            float SynEq[][MNT],
            int SynType[][MAX_SYN]) {
   int nt[MNT];
   int c,c2;
   for(c=0; c<N_C; c++) {
      for(c2=0; c2<MNT; c2++)
         nt[c2] = 0;
      for(c2=0; c2<SynNum[c]; c2++)
         nt[SynType[c][c2]]++;
      for(c2=0; c2<MNT; c2++)
         if(nt[c2] > 0) SynEq[c][c2] = 1.0 / nt[c2];
         //else set to 1.0 in cold-start
      }
}//setEqu


void plot_n(int x, int y, int size, int color) {
   int c,c2;
   for(c=0; c<size; c++)
      for(c2=0; c2<size; c2++)
         putpixel(x+c2,y+c,color);
}//plot_n


void plot_4(int x, int y, int color) {
   if(getpixel(x,y) == color) return;
   putpixel(x  ,y  ,color);
   putpixel(x  ,y+1,color);
   putpixel(x+1,y  ,color);
   putpixel(x+1,y+1,color);
}//plot_4


void plot_depo(int State[],int LastTir[]) {
   int y,color;
   for(y=0; y<N_C; y++) {
      if(State[y]==0)                     //on depo.
         if(LastTir[y]<R_P_SPAN)          //plot wavelength??
            color = GREY[SCR][LastTir[y]];   //if low frec..
         else color = GREY[SCR][R_P_SPAN-1]; //dark
      else color = BLACK;
      plot_4(TimePos*2,y*2+WO_PO_Y,color); //no depo: erase
      }
}//plot_depo


void display_up(int Type   [],        //prototype
                float NtInp1[][MNT],
                float NtInp2[][MNT],
                float SynEq [][MNT]) ;


void plot_waves(int Type   [],
                int State  [],
                float NtInp1[][MNT],
                float NtInp2[][MNT],
                float SynEq [][MNT]) {
   const int tRUNCATE = 30,
             wIDE = 250; //depolarization window is 250 wide
   static int eeg[wIDE][WAVES],
              wave_pos[WAVES];
   int c,c2,wave[WAVES],wc; // wc: waves counter
   if(eeg[0][0] == 0)  { //static vars set to 0 in declaration
      wave_pos[0] = getmaxy()-24;
      wave_pos[1] = getmaxy()-16;
      wave_pos[2] = getmaxy()-8;
      wave_pos[3] = getmaxy();
      for(c=0; c<wIDE; c++)
         for(c2=0; c2<WAVES; eeg[c][c2] = wave_pos[c2], c2++);
      }
   if (++TimePos >= wIDE) {
      TimePos = 0;
      if (DispUp == 'B') display_up(Type,NtInp1,NtInp2,SynEq); }
   for(wc=0; wc<WAVES; wc++) {
      for(wave[wc]=0, c=EegB[wc]; c<=EegE[wc]; c++)
         if(State[c] == 0) wave[wc]++;
      if (wave[wc] > tRUNCATE) wave[wc] = tRUNCATE;
      }
   for(c=0; c<WAVES; c++) {
      setcolor(BLACK);                                //erase old lines
      line(TimePos *2,   eeg[TimePos][c],
           TimePos *2+2, eeg[TimePos+1][c]);
      eeg[TimePos][c] = wave_pos[c] - (wave[c] * 2); //compute new values
      setcolor(NtCol[SCR][c+2]);
      line(TimePos *2,  eeg[TimePos][c],
           TimePos *2-2,eeg[TimePos-1][c]);         //draw new lines
      }
}//plot_waves


void plot_nei(int State [],
              int SynNum[],
              int SynType [][MAX_SYN],
              int *Syn    [][MAX_SYN],
              float SynWei[][MAX_SYN]) {
   const double scale = 10.0;
   int c,c2,
       tempState[N_C],
       syn_size;
   for(c=0,gotoxy(1,12); c<13; c++)
      printf("\n                                                  ");
   for(c=0; c<N_C; tempState[c] = State[c], State[c] = c, c++);
   for(c=0; c<N_C; c++) //plot synaptic position & type
      for(c2=0; c2<SynNum[c]; c2++)
         if( (SynType[c][c2] > 0) && (SynType[c][c2] <= MNT) )
            plot_4(c*2, *Syn[c][c2]*2+WO_PO_Y, NtCol[SCR][SynType[c][c2]]);
            else plot_4(c*2, *Syn[c][c2]*2+WO_PO_Y,143);//143 blinking white
   for(c=0; c<N_C; c++) //plot weights??
      for(c2=0; c2<SynNum[c]; c2++){
         syn_size = (int) (log(SynWei[c][c2])*scale);
         if(syn_size<16)
            plot_4( (c*2)+(N_C*2),
                    *Syn[c][c2]*2+WO_PO_Y,
                    GREY[SCR][15-syn_size]);
            else plot_4((c*2)+(N_C*2), *Syn[c][c2]*2+WO_PO_Y, 143);//bl.white
         }
   for(c=0; c<N_C; State[c] = tempState[c], c++);
}//plot_nei


void plot_wei(int Type  [],
              int State [],
              int SynNum[],
              int SynType [][MAX_SYN],
              int *Syn    [][MAX_SYN],
              float SynWei[][MAX_SYN]) {
const int top    =   0,
          defa   = 100, //position of default SynWei[][] (1.0)
          bottom = 394, //394 allows cleaning screen with 25 clean_line()
          s      =   6; //s-1 = spacing pixels between cells
   int cell = 0,
       c,c2,
       tempState[N_C];
   for(c=1; c<=25; clean_line(c), c++) ;
   for(c=0; c<N_C; tempState[c] = State[c], State[c] = c, c++) ;
   setcolor(DARKGRAY);           //draw  lines at powers of two
   line(0, (int)(log( 1.0)*defa+defa), N_C*s, (int)(log( 1.0)*defa+defa));
   line(0, (int)(log( 2.0)*defa+defa), N_C*s, (int)(log( 2.0)*defa+defa));
   line(0, (int)(log( 4.0)*defa+defa), N_C*s, (int)(log( 4.0)*defa+defa));
   line(0, (int)(log( 8.0)*defa+defa), N_C*s, (int)(log( 8.0)*defa+defa));
   line(0, (int)(log(16.0)*defa+defa), N_C*s, (int)(log(16.0)*defa+defa));
   for(c=0; c<N_C; c++) {  //show all synapses of all cells
      plot_4(c*s,   top,NtCol[SCR][Type[c]]);
      plot_4(c*s,bottom,NtCol[SCR][Type[c]]);
      for(c2=0; c2<SynNum[c]; c2++)
         if(SynWei[c][c2] > 1.0) {  //plot every synapse over 1.0
            //setcolor(NtCol[SCR][SynType[c][c2]]);
            if(SynWei[c][c2] < 16.0)
               //line(c*s,top+3,*Syn[c][c2]*s,
                    //(int)(log(SynWei[c][c2])*defa+defa));
               putpixel(*Syn[c][c2]*s,(int)(log(SynWei[c][c2])*defa+defa),
                        NtCol[SCR][SynType[c][c2]] );
               else line(c*s,top+3,*Syn[c][c2]*s,bottom);
            }
      }
   do { //show a single cell
      if(cell < 0) cell += N_C; //wrap borders
         else if(cell >= N_C) cell -= N_C;
      gotoxy(76,1); printf("%3d",cell); //show number of cell..
      plot_4(cell*s,   top,WHITE);      //..and position
      plot_4(cell*s,bottom,WHITE);
      setcolor(DARKGRAY);
      for(c=0; c<SynNum[cell]; c++) { //plot this cell's synapses
         plot_4(*Syn[cell][c]*s, bottom+4, NtCol[SCR][SynType[cell][c]]);
         if(SynWei[cell][c] > 1.0)    //and a line for those grown
            if(SynWei[cell][c] < 16.0)
               line(cell*s, top+3,*Syn[cell][c] * s,
                    (int)(log(SynWei[cell][c]) * defa + defa) );
               else line(cell*s,top+3,*Syn[cell][c]*s,bottom);
         }
      if(getch() == 0) {
         plot_4(cell*s,   top,NtCol[SCR][Type[cell]]);
         plot_4(cell*s,bottom,NtCol[SCR][Type[cell]]);
         for(c=0; c<SynNum[cell]; c++) {
            plot_4(*Syn[cell][c]*s, bottom+4, BLACK); //erase !grown syn.
            if(SynWei[cell][c] > 1.0) {
               setcolor(NtCol[SCR][SynType[cell][c]]);//normal color of grown
               if(SynWei[cell][c] < 16.0)
                  line(cell*s, top+3,*Syn[cell][c] * s,
                     (int)(log(SynWei[cell][c]) * defa + defa) );
                  else line(cell*s,top+3,*Syn[cell][c]*s,bottom);
               }
            }
         switch(getch()) {
            case 72: cell -= 10; break;
            case 77: cell++;     break;
            case 80: cell += 10; break;
            case 75: cell--;     break;
            }
         }
         else break;
      }
      while (1);
   for(c=0; c<N_C; State[c] = tempState[c], c++);
   for(c=1; c<=25; clean_line(c), c++) ;
}//plot_wei


void plot_frog(int x, int y) {
   x += (Pox-2)*WS;
   y += (Poy-2)*WS;
   plot_n(x, y, WS,WHITE); //"body"
   switch(Direc) {         //"eyes"
      case 'n':plot_4(   x+1,    y+1, BLACK);
               plot_4(x+WS-3,    y+1, BLACK);
               break;
      case 's':plot_4(   x+1, y+WS-3, BLACK);
               plot_4(x+WS-3, y+WS-3, BLACK);
               break;
      case 'e':plot_4(x+WS-3,    y+1, BLACK);
               plot_4(x+WS-3, y+WS-3, BLACK);
               break;
      case 'w':plot_4(   x+1,    y+1, BLACK);
               plot_4(   x+1, y+WS-3, BLACK);
               break;
      }
}//plot_frog


void plot_World(void) {
   int wo_col[8] = //world color
      {RED,GREEN,LIGHTBLUE,YELLOW,CYAN,MAGENTA,WHITE,BLACK},
       x,y;
   for(y=0; y<WO_SZ_Y-4; y++)
      for(x=0; x<WO_SZ_X-4; x++)
         plot_n(WO_PO_X+(x*WS),WO_PO_Y+(y*WS),WS,wo_col[World[y+2][x+2]]);
   plot_frog(WO_PO_X, WO_PO_Y);
   for(x=0; x<16; x++) //show spectrum
      plot_n(WO_PO_X+(x*WS),WO_PO_Y+((WO_SZ_Y-3)*WS),WS,GREY[SCR][x]);
}//plot_World


void show_LMRG(void) { /*show Left Move Right Generations*/
   double temp = ((Left + Move + Right) / 3.0) * MOTOR_TH;
   gotoxy(70,22);
   if(temp < 10.0) printf(" "); printf("%1.2lf",temp);
   gotoxy(64,23);
   if(Left < 10.0) printf(" ");
   if(Left > 99.9) printf(" **.*"); else printf(" %3.1lf",Left );
   if(Move < 10.0) printf(" ");
   if(Move > 99.9) printf(" **.*"); else printf(" %3.1lf",Move );
   if(Right < 10.0) printf(" ");
   if(Right > 99.9) printf(" **.*"); else printf(" %3.1lf",Right);
   gotoxy(67,25); printf(" %6ld",Gen);
} //show_LMRG


void show_syn(int cell, int begin, int tempi,
              int   SynType[][MAX_SYN],
              int   *Syn   [][MAX_SYN],
              float SynWei [][MAX_SYN],
              float SyNt1  [][MAX_SYN],
              float SyNt2  [][MAX_SYN]) {
   int c;
   printf("\n\ntyp");
   for(c=begin; c<tempi; c++)
      printf("%6d",SynType[cell][c]);
   printf("\npos");
   for(c=begin; c<tempi; c++)
      printf("%6d",*Syn[cell][c]);
   printf("\npri");
   for(c=begin; c<tempi; c++)
      printf("%6ld",(long) (SyNt1[cell][c] * FT) );
   printf("\nsec");
   for(c=begin; c<tempi; c++)
      printf("%6ld",(long) (SyNt2[cell][c] * FT) );
   printf("\nwei");
   for(c=begin; c<tempi; c++)
      printf("%6ld",(long)(SynWei[cell][c] * FT) );
}//show_syn


int change_cell() {
   int temp,c;
   gotoxy( 9,2); printf("91");
   gotoxy(22,2); printf("92");
   gotoxy(38,2); printf("93");
   gotoxy(70,2); printf("94");
   for(c=0; c<MNT; c++) {gotoxy(c*9+11,4); printf("%d1",c); }
   for(c=0; c<MNT; c++) {gotoxy(c*9+11,5); printf("%d2",c); }
   temp = ask_int("type the digits after '' ",-1);
   return(temp);
}//change_cell


void show_cell(int Type   [],
               int State  [],
               int LastTir[],
               int SynNum [],
               float AuxInp[],
               float NtInp1 [][MNT],
               float NtInp2 [][MNT],
               float SynEq  [][MNT],
               int   SynType[][MAX_SYN],
               int   *Syn   [][MAX_SYN],
               float SynWei [][MAX_SYN],
               float SyNt1  [][MAX_SYN],
               float SyNt2  [][MAX_SYN]) {
   int cell = 0, c = 0, tempi = 0, returned = 0,
       tempState[N_C];
   char dumch = '-';
   for(c=0; c<N_C; c++) { //to know which cell is pointed by *Syn[][]
      tempState[c] = State[c];
      State[c] = c;
      }
   do {
      for(c=1; c<=25; clean_line(c++)) ;
      gotoxy(1,1);
      if(cell < 0) cell += N_C;
         else if(cell >= N_C) cell -= N_C;
      printf("cell %2d[cursor]     cHanges    [any key] exit",cell);
      printf("\n  type %d     state%3d     w.length%3d",
             Type[cell],tempState[cell],LastTir[cell]);
      printf("     syn-num %2d    aux-inp %1.3f",
             SynNum[cell],AuxInp[cell]);
      printf("\n     [0]      [1]      [2]      [3]");
      printf("      [4]      [5]      [6]      [7]");
      printf("\npri ");
      for(c=0; c<MNT; c++)
         printf(" %2.3f   ",NtInp1[cell][c]*SynEq[cell][c]);
      printf("\nsec ");
      for(c=0; c<MNT; c++)
         printf(" %2.3f   ",NtInp2[cell][c]*SynEq[cell][c]);
      if(SynNum[cell] > 0) {
         tempi = (SynNum[cell] > 12) ? 12 : SynNum[cell];
         show_syn(cell, 0,tempi,SynType,Syn,SynWei,SyNt1,SyNt2);
         }
      if(SynNum[cell] > 12) {
         tempi = (SynNum[cell] > 24) ? 24 : SynNum[cell];
         show_syn(cell,12,tempi,SynType,Syn,SynWei,SyNt1,SyNt2);
         }
      if(SynNum[cell] > 24) {
         tempi = (SynNum[cell] > 36) ? 36 : SynNum[cell];
         show_syn(cell,24,tempi,SynType,Syn,SynWei,SyNt1,SyNt2);
         }
      dumch = getch();
      if(!dumch)
         switch(getch()) {
            case 72: cell -= 10; break;
            case 77: cell++;     break;
            case 80: cell += 10; break;
            case 75: cell--;     break;
            }
         else if(dumch=='h')
            switch(returned=change_cell()) {
               case 01: case 11: case 21: case 31:
               case 41: case 51: case 61: case 71:
                       NtInp1[cell][returned/10] = ask_double("primary NT",
                       NtInp1[cell][returned/10] * SynEq[cell][returned/10])
                       / SynEq[cell][returned/10];
                       break;
               case 02: case 12: case 22: case 32:
               case 42: case 52: case 62: case 72:
                       NtInp2[cell][returned/10] = ask_double("secondary NT",
                       NtInp2[cell][returned/10] * SynEq[cell][returned/10])
                       / SynEq[cell][returned/10];
                       break;
               case 91:Type[cell] = ask_int("cell type:",Type[cell]);
                       break;
               case 92:tempState[cell] =
                          ask_int("cell state:",tempState[cell]);
                       break;
               case 93:LastTir[cell] = ask_int("wavelength:",LastTir[cell]);
                       break;
               case 94:AuxInp[cell] = ask_double("aux. input:",AuxInp[cell]);
                       break;
               default:clean_line(1);
                       printf("DOES NOT COMPUTE");
                       delay(1000);
                       menu_line();
               }
            else break;
      }
      while (1);
   for(c=0; c<N_C; c++) State[c] = tempState[c];
   for(c=1; c<=25; clean_line(c++)) ;
}//show_cell


void def_menu(void) {  //default menu, parameters
   int c;
   clean_upside();
   gotoxy(1,2);
   for(c=0; c<MNT; c++) {
      printf("\n%d)",c);
      printf("   %3d",HL_PRI[c]);
      printf("      %3d",HL_TRA[c]);
      printf("      %3d",HL_SEC[c]);
      printf("     %1.3lf",NT_TH[c]);
      if (NT_PO[c] >= 0.0) printf(" ");
      printf("   %1.3f",NT_PO[c]);
      switch (c) {
         case 0:printf("    %1.3lf energy     ",ATP);      break;
         case 1:printf("    %1.3lf lactic     ",LAC);      break;
         case 2:printf("    %1.3lf aux inp    ",STI);      break;
         case 3:if (!SCR) printf("          color      ");
                   else   printf("       black&white   ");
                break;
         case 5:printf("    %1.3lf motor thre ",MOTOR_TH); break;
         case 4:printf("     %3d  -life aux ",HL_AUX);   break;
         }
      }
   printf("\nNT   life    trans    l.s.   thres    power");
   menu_line();
}//def_menu


void bars(int inc,
          int Type[],
          float NtInp1[][MNT],
          float NtInp2[][MNT],
          float SynEq [][MNT]) {
   const int y_margin = 16;
   float sz = 40.0;
   int c,c2,
       sca = 3,
       level[MNT] = {  0, 80,160,240,320,400,480,560};
   if(inc>1) clean_upside();
      else for(c=1; c<21; clean_line(c++)) ;
   setcolor(LIGHTGRAY);
   for(c=0; c<MNT; c++) {
   //log 0.5:-0.69315  1:0  2:0.69315  3:1.09861  4:1.38629  5:1.60944
      line((int)(level[c]+(log(1.0*1.0+1.0)*sz)), y_margin,
           level[c]+(int)(0.69315 * sz),          y_margin+10);
      line((int)(level[c]+(log(2.0*1.0+1.0)*sz)), y_margin,
           level[c]+(int)(1.09861 * sz),          y_margin+10);
      line((int)(level[c]+(log(4.0*1.0+1.0)*sz)), y_margin,
           level[c]+(int)(1.60944 * sz),          y_margin+10);
      }
   for(c=0; c<N_C; c+=inc)
      for (c2=0; c2<MNT; c2++) {
         if(c2==0) setcolor(NtCol[SCR][Type[c]]);
            else setcolor(NtCol[SCR][c2]);
         line( level[c2],
            (c/inc) * sca + y_margin,
            (int)(level[c2]+(log(NtInp1[c][c2]*SynEq[c][c2]+1.0)*sz)),
            (c/inc) * sca + y_margin);
         putpixel((int)(level[c2]+(log(NtInp2[c][c2]*SynEq[c][c2]+1.0)*sz)),
            (c/inc) * sca + y_margin + 1,15);
         }
   if(inc==1) {getch(); for(c=1; c<21; clean_line(c++)) ; }
}//bars


void show_table(int x,int y) {
   int c,c2;
   clean_upside();
   switch (DispUp) {
      case 'F':for(printf("syn type"), c=0; c<y; c++) {
                  printf("\n%d) ",c);
                  for(c2=0; c2<x; c2++)
                     if(FOR[c][c2] == 0.0) printf(" 0.0  ");
                        else printf(" %1.3lf",FOR[c][c2]);
                  }
               break;
      case 'A':for(printf("cell type"), c=0; c<y; c++) {
                  printf("\n%d) ",c);
                  for(c2=0; c2<x; c2++)
                     if(ACTIVE[c][c2] == 0) printf("  0  ");
                        else printf(" %2d  ",ACTIVE[c][c2]);
                  }
               break;
      case 'C':for(printf("catalizing NT"), c=0; c<y; c++) {
                  printf("\n%d) ",c);
                  for(c2=0; c2<x; c2++)
                     if(CAT[c][c2] == 0.0) printf(" 0.0  ");
                        else printf(" %1.3lf",CAT[c][c2]);
                  }
               break;
      case 'P':for(printf("y:NT    x:time-step"), c=0; c<y; c++) {
                  printf("\n%d)",c);
                  for(c2=0; c2<x; c2++)
                     if(REF_PERn[c][c2] == 0.0) printf(" 0.0 ");
                        else printf(" %1.2lf",REF_PERn[c][c2]);
                  }
               break;
      }
   if(DispUp == 'P') //??
      for(printf("\n  "), c=0; c<x;c++)
         if(c<10) printf(" [0%d]",c);
            else printf(" [%2d]",c);
      else if((DispUp == 'F')||(DispUp == 'C'))
         for(printf("\n  "), c=0; c<x;c++) printf("  [%d] ",c);
         else
         for(printf("\n  "), c=0; c<x;c++) printf("  [%d]",c);
   switch(DispUp) {      //??
      case 'F':printf(" reacting-NT "); break;
      case 'A':printf(" growth-active synaptic types"); break;
      case 'C':printf(" catalized NT"); break;
      //case 'P':printf("time"); break;
      }
}//show_table


void display_up(int Type   [],
                float NtInp1[][MNT],
                float NtInp2[][MNT],
                float SynEq [][MNT]) {
   switch(DispUp) {
      case 'B':bars(2,Type,NtInp1,NtInp2,SynEq); break;
      case 'M':def_menu(); break;
      case 'F':show_table(MNT,MNT); break;
      case 'A':show_table(MNT,MNT); break; //grow_des(); break;
      case 'C':show_table(MNT,MNT); break;
      case 'P':show_table(R_P_SPAN,MNT); break;
      default:printf("DispUp invalid"); getch;
      }
}//display_up


void change_params(void) {
int which,hiby,c,c2;
   which = hiby = 0;
   gotoxy(1,2);
   for(c=0; c<MNT; c++)
      for (c2=1; c2<6; c2++) {
         gotoxy(c2*9,c+3);
         printf("%d%d",c,c2);
         }
   for(c=81,c2=3; c<87; c++,c2++) {
      gotoxy(67,c2);
      printf("%d",c);
      }
   clean_line(1);
   which = ask_int("please type the digits after '' ",-1);
   hiby = (int) which / 10;
   switch(which) {
      case 01: case 11: case 21: case 31: case 41: case 51: case 61: case 71:
              HL_PRI[hiby] = ask_int("primary NT -life",HL_PRI[hiby]);
              HL_P[hiby] = half_life(HL_PRI[hiby]);
              break;
      case 02: case 12: case 22: case 32: case 42: case 52: case 62: case 72:
              HL_TRA[hiby] = ask_int("pri->sec transformation",HL_TRA[hiby]);
              HL_T[hiby] = 1.0 - half_life(HL_TRA[hiby]);
              break;
      case 03: case 13: case 23: case 33: case 43: case 53: case 63: case 73:
              HL_SEC[hiby] = ask_int("sec. NT life ",HL_SEC[hiby]);
              HL_S[hiby] = half_life(HL_SEC[hiby]);
              break;
      case 04: case 14: case 24: case 34: case 44: case 54: case 64: case 74:
              NT_TH[hiby] = ask_double("NT ThresHold ",NT_TH[hiby]);
              break;
      case 05: case 15: case 25: case 35: case 45: case 55: case 65: case 75:
              NT_PO[hiby] = ask_double("NT POwer ",NT_PO[hiby]);
              break;
      case 81:ATP = ask_double("energy (ATP)",ATP); break;
      case 82:LAC = ask_double("fatigue (lactic acid)",LAC); break;
      case 83:STI = ask_double("sensor input",STI); break;
      case 84:SCR = (SCR) ? 0 : 1; plot_World(); break;
      case 85:HL_AUX = ask_int("auxiliary input half-life",HL_AUX);
              HL_A = half_life(HL_AUX);
              break;
      case 86:MOTOR_TH = ask_double("motor threshold",MOTOR_TH); break;
      default:clean_line(1);
              printf("DOES NOT COMPUTE");
              delay(1000);
              //menu_line();
      }
   menu_line();
}//change_params


void change_for() {
   int y,x;
   y = ask_int("          syn type ?",-1);
   x = ask_int("          NT ?",-1);
   if( (y>1) && (y<MNT) && (x>=0) && (x<MNT) )
      FOR[y][x] = ask_double("chemical activity ",FOR[y][x]);
   else {clean_line(1);printf("DOES NOT COMPUTE");delay(1000);menu_line();}
}//change_for
void change_acti() {
   int y,x;
   y = ask_int("          cell type ?",-1);
   x = ask_int("          syn type  ?",-1);
   if( (y>1) && (y<MNT) && (x>1) && (x<MNT) )
      ACTIVE[y][x] = (ACTIVE[y][x]) ? 0 : 1;
   else {clean_line(1);printf("DOES NOT COMPUTE");delay(1000);menu_line();}
}//change_acti
void change_cat() {
   int y,x;
   y = ask_int("          catalizing  sec. NT ?",-1);
   x = ask_int("          catalized sec. NT ?",-1);
   if( (y>=0) && (y<MNT) && (x>=0) && (x<MNT) && (y!=x) )
      CAT[y][x] = ask_double("catalization activity ",CAT[y][x]);
   else {clean_line(1);printf("DOES NOT COMPUTE");delay(1000);menu_line();}
}//change_cat
void change_ref_per() {
   int y,x;
   y = ask_int("          NT ?",-1);
   x = ask_int("          time step ?",-1);
   if( (y>=0) && (y<MNT) && (x>=0) && (x<R_P_SPAN) )
      REF_PERn[y][x] = ask_double(
         "NT proportion crossing cleft ",REF_PERn[y][x]);
   else {clean_line(1);printf("DOES NOT COMPUTE");delay(1000);menu_line();}
}//change_ref_per


char motor_navigation(int LastTir[], float AuxInp[], char reset) ;

void warm_reset(int State  [],
                int LastTir[],
                float AuxInp[],
                float NtInp1 [][MNT],
                float NtInp2 [][MNT],
                float SynWei [][MAX_SYN],
                float SyNt1  [][MAX_SYN],
                float SyNt2  [][MAX_SYN]) {
   int dum_i = 0;
   clean_line(1);
   printf(" [RETURN] default   [SPACE] ask seed   [q] quit");
   switch (getch()) {
      case 'q':break;
      case 13 :SEED = 345;
               reset(State,LastTir,AuxInp,NtInp1,NtInp2,
                     SynWei,SyNt1,SyNt2);
               motor_navigation(LastTir,AuxInp,1);
               break;
      case ' ':clean_line(1);
               printf("seed? ");
               scanf("%d",&dum_i);
               gotoxy(40,1);
               printf("readed: %d ",dum_i);
               delay(1000);
               SEED = dum_i;
               reset(State,LastTir,AuxInp,NtInp1,NtInp2,
                     SynWei,SyNt1,SyNt2);
      }
   plot_World();
   show_LMRG();
}//warm_reset;


long menu(int Type   [],
          int State  [],
          int LastTir[],
          int SynNum [],
          float AuxInp[],
          float NtInp1 [][MNT],
          float NtInp2 [][MNT],
          float SynEq  [][MNT],
          int   SynType[][MAX_SYN],
          int   *Syn   [][MAX_SYN],
          float SynWei [][MAX_SYN],
          float SyNt1  [][MAX_SYN],
          float SyNt2  [][MAX_SYN]) {
   char tempch;
   do{
      tempch = '-';
      menu_line();
      display_up(Type,NtInp1,NtInp2,SynEq);
      tempch = getch();
      switch( tempch ) {
         case '0':return(1);           //1 Generation
         case '1':return(10);          //10   Gen.
         case '2':return(250);         //250  Gen.
         case '3':return(2147483647l); //2147 483 647 biggest signed long int

         case 'n':plot_nei(State,SynNum,SynType,Syn,SynWei); break;
         case 'w':plot_wei(Type,State,SynNum,SynType,Syn,SynWei);
                  plot_World();
                  show_LMRG();
                  break;
         case 'B':bars(1,Type,NtInp1,NtInp2,SynEq);
                  plot_World();
                  show_LMRG();
                  break;
         case 'b':DispUp = 'B'; break;
         case 'e':show_cell(Type,State,LastTir,SynNum,
                            AuxInp,
                            NtInp1,NtInp2,SynEq,
                            SynType,Syn,
                            SynWei,SyNt1,SyNt2);
                  plot_World();
                  show_LMRG();
                  break;
         case 'h':switch (DispUp) {
                     case 'M':change_params();  break;
                     case 'F':change_for();     break;
                     case 'A':change_acti();    break;
                     case 'C':change_cat();     break;
                     case 'P':change_ref_per(); break;
                     }
                  break;
         case 'm':DispUp = 'M'; break;
         case 'f':DispUp = 'F'; break;
         case 'a':DispUp = 'A'; break;
         case 'c':DispUp = 'C'; break;
         case 'p':DispUp = 'P'; break;
         case 'r':warm_reset(State,LastTir,AuxInp,
                             NtInp1,NtInp2,SynWei,SyNt1,SyNt2);
                  break;
         case 'v':params_file(0); break; //save parameters
         case  27:return(0);     //ascii 27 == ESC, terminate program
         }
      }while(1); //??
}//menu


void retina(float AuxInp[]) {
//24 cells: 3 primary colors * binary intensity == 8 combinations
const int retsize  = 24,
          spectrum =  8;
   static int inp [ spectrum ] [ retsize ] = {
      1,0,0, 1,0,0, 1,0,0, 1,0,0, 1,0,0, 1,0,0, 1,0,0, 1,0,0,  //red
      0,1,0, 0,1,0, 0,1,0, 0,1,0, 0,1,0, 0,1,0, 0,1,0, 0,1,0,  //green
      0,0,1, 0,0,1, 0,0,1, 0,0,1, 0,0,1, 0,0,1, 0,0,1, 0,0,1,  //blue
      1,1,0, 1,1,0, 1,1,0, 1,1,0, 1,1,0, 1,1,0, 1,1,0, 1,1,0,  //yellow
      0,1,1, 0,1,1, 0,1,1, 0,1,1, 0,1,1, 0,1,1, 0,1,1, 0,1,1,  //cyan
      1,0,1, 1,0,1, 1,0,1, 1,0,1, 1,0,1, 1,0,1, 1,0,1, 1,0,1,  //magenta
      1,1,1, 1,1,1, 1,1,1, 1,1,1, 1,1,1, 1,1,1, 1,1,1, 1,1,1,  //white
      0,0,0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, 0,0,0}; //black
   static int looks [ retsize ] = //which pixel sees each cell
      {0,0,0, 1,1,1, 2,2,2, 3,3,3, 4,4,4, 5,5,5, 6,6,6, 7,7,7};
   int c,pixel[8] = {0,0,0,0,0,0,0,0}; //content of pixels
   switch (Direc) {
      case 'n':pixel[0] = World[Poy-2][Pox-1]; //up left
               pixel[1] = World[Poy-2][Pox  ]; //up
               pixel[2] = World[Poy-2][Pox+1]; //up right
               pixel[3] = World[Poy-1][Pox-1]; //middle left
               pixel[4] = World[Poy-1][Pox  ]; //middle
               pixel[5] = World[Poy-1][Pox+1]; //middle right
               pixel[6] = World[Poy  ][Pox-1]; //left
               pixel[7] = World[Poy  ][Pox+1]; //right
               for(c=NOR; c<EAS; c++) AuxInp[c] += STI; //direction
               break;
      case 'e':pixel[0] = World[Poy-1][Pox+2];
               pixel[1] = World[Poy  ][Pox+2];
               pixel[2] = World[Poy+1][Pox+2];
               pixel[3] = World[Poy-1][Pox+1];
               pixel[4] = World[Poy  ][Pox+1];
               pixel[5] = World[Poy+1][Pox+1];
               pixel[6] = World[Poy-1][Pox  ];
               pixel[7] = World[Poy+1][Pox  ];
               for(c=EAS; c<SOU; c++) AuxInp[c] += STI;
               break;
      case 's':pixel[0] = World[Poy+2][Pox+1];
               pixel[1] = World[Poy+2][Pox  ];
               pixel[2] = World[Poy+2][Pox-1];
               pixel[3] = World[Poy+1][Pox+1];
               pixel[4] = World[Poy+1][Pox  ];
               pixel[5] = World[Poy+1][Pox-1];
               pixel[6] = World[Poy  ][Pox+1];
               pixel[7] = World[Poy  ][Pox-1];
               for(c=SOU; c<WES; c++) AuxInp[c] += STI;
               break;
      case 'w':pixel[0] = World[Poy+1][Pox-2];
               pixel[1] = World[Poy  ][Pox-2];
               pixel[2] = World[Poy-1][Pox-2];
               pixel[3] = World[Poy+1][Pox-1];
               pixel[4] = World[Poy  ][Pox-1];
               pixel[5] = World[Poy-1][Pox-1];
               pixel[6] = World[Poy+1][Pox  ];
	            pixel[7] = World[Poy-1][Pox  ];
               for(c=WES; c<LEF; c++) AuxInp[c] += STI;
               break;
      }
   for(c=0; c<retsize; c++) //feed retina's input to AuxInp
      if ( inp[ pixel[ looks[c] ] ] [c] ) AuxInp[c] += STI;
}/*retina*/


char motor_navigation(int LastTir[], float AuxInp[], char reset) {
const double reinforce = 4.0; //magic??
static int pause = 100; //wait for cold-start/reset frog to warm-up
double mean = 0.0;
int c, opox,opoy,  //old position in X and Y axis
    wo_col[8] =    //world color, like in plot-World
       {RED,GREEN,LIGHTBLUE,YELLOW,CYAN,MAGENTA,WHITE,BLACK};
char status = '\0'; //l:left r:right m:move c:crash \0:no activity
   if(reset) pause = 100;
   if(pause > 0) {
      --pause;
      return(status);
      }
   Left  = 0.0; //global so show_LMRG() can show 'em
   Move  = 0.0;
   Right = 0.0;
   opox = Pox;  //store actual position in case it tries to move and crashes
   opoy = Poy;
   for(c=LEF; c<MOV; Left  += LastTir[c], c++) ;
   Left /= MOV - LEF;
   for(c=MOV; c<RIG; Move  += LastTir[c], c++) ;
   Move /= RIG - MOV;
   for(c=RIG; c<FREE; Right += LastTir[c], c++) ;
   Right /= FREE - RIG;
   mean = (Left + Move + Right) / 3.0;
                           //order changed: move has priority
   if ( (status =='\0') && (Move < mean * MOTOR_TH) ) {
      status = 'm';
      switch (Direc) {
         case 'n': --Poy; break;
         case 'e': ++Pox; break;
         case 's': ++Poy; break;
         case 'w': --Pox; break;
         }
      }
   if ( (status =='\0') && (Left < mean * MOTOR_TH) ) {
      status = 'l';
      switch (Direc) {
         case 'n': Direc = 'w'; break;
         case 'e': Direc = 'n'; break;
         case 's': Direc = 'e'; break;
         case 'w': Direc = 's'; break;
         }
      }
   if ( (status =='\0') && (Right < mean * MOTOR_TH) ) {
      status = 'r';
      switch (Direc) {
         case 'n': Direc = 'e'; break;
         case 'e': Direc = 's'; break;
         case 's': Direc = 'w'; break;
         case 'w': Direc = 'n'; break;
         }
      }
   if (World[Poy][Pox] == 0) { //hits a rock!
      status = 'c';
      Poy = opoy; //return frog to last position
      Pox = opox;
      }
   if(status) {
      switch(status) {
         case 'l':
         case 'r':plot_frog(WO_PO_X,WO_PO_Y); break;
         case 'm':if(World[Poy][Pox] > 1) { //any food?
                     --World[Poy][Pox];     //food gets eaten, dose for PLEA
                     for(c=PLEAb; c<=PLEAe; AuxInp[c] += reinforce, c++) ; }
                  plot_n(WO_PO_X+((opox-2)*WS),WO_PO_Y+((opoy-2)*WS),
                         WS,wo_col[World[opoy][opox]]);
                  plot_frog(WO_PO_X,WO_PO_Y);
                  break;
         case 'c':for(c=PAINb; c<=PAINe; AuxInp[c] += reinforce, c++) ;
                  break;
         default :printf("\n\007motor/navigation error"); getch();
         }
      pause += (R_P_SPAN*3); //after moving, frog is inactive for a while
      }
   return(status);
}//motor_navigation


/* ALL SYNAPSES ARE CREATED EQUAL, ALL SYNAPSES MUST BE JUDGED EQUAL */
void compute(int   Type   [],
             int   State  [],
             int   LastTir[],
             int   SynNum [],
             float AuxInp [],
             float NtInp1 [][MNT],
             float NtInp2 [][MNT],
             float SynEq  [][MNT],
             int   SynType[][MAX_SYN],
             int   *Syn   [][MAX_SYN],
             float SynWei [][MAX_SYN],
             float SyNt1  [][MAX_SYN],
             float SyNt2  [][MAX_SYN]) {
   double tot_inp = 0.0, //total equalized NTs input (formula)
          synAct  = 0.0, //synaptic activity (growth)
          des[MNT];      //local NT destruction, resets on CATALYSIS
   int c,c2,c3,
       ref_per,          //local Refractory Period (to use States > R_P_SPAN)
       cell_typ,         //local cell type
       syn_typ,          //local Synaptic Type
       next_state[N_C];  //next_state[] defined in formula

   for(c=0; c<N_C; c++) { //for all cells
      cell_typ = Type[c];
      ref_per = (State[c] < R_P_SPAN) ? State[c] : R_P_SPAN-1 ;
//INPUT   read input from all synapses in this cell
        //if neig. depolarizes (state 0), add pri-NT..
        //..to both syn. store & cell's total of that NT-type
      for(c2=0; c2<SynNum[c]; c2++) {
         syn_typ = SynType[c][c2];
         if(*Syn[c][c2] == 0) { //neighbor depolarizes?
            //add syn. weight filtered through ref. per.
            NtInp1[c][syn_typ] += SynWei[c][c2] * REF_PERn[syn_typ][ref_per];
            SyNt1 [c][c2]  += SynWei[c][c2] * REF_PERn[syn_typ][ref_per];
            }
         SyNt2[c][c2] += SyNt1[c][c2] * HL_T[syn_typ]; //syn  pri->sec
         }
      NtInp1[c][0] += ATP * REF_PERn[0][ref_per]; //"Na pump" (!syn[0]||[1])
                                   //there's energy to fire?     (magic??)
      if((tot_inp = NtInp1[c][0] * REF_PERc[0][ref_per] * NT_PO[0]) > 0.5) {
         tot_inp += AuxInp[c];     //read external input
         for(c2=1; c2<MNT; c2++)   //read other NTs
            tot_inp += (NtInp1[c][c2] * SynEq[c][c2] * NT_PO[c2]);
         }
      /*************** FORMULA ****************/
      if( tot_inp >= NT_TH[cell_typ] ) { //total input > threshold?
         next_state[c] = 0;              //depolarize!
         LastTir[c] = State[c];          //update frequency
         NtInp1[c][0] -= LAC;            //turn a little ATP..
         NtInp1[c][1] += LAC;            //..into LAC
         }
         else next_state[c] = State[c] + 1;
                                                       //cell pri->sec
      for(c2=0; c2<MNT; NtInp2[c][c2] += NtInp1[c][c2] * HL_T[c2], c2++) ;
//CATALYSIS AND DECAY
      //enzime & blocker interactions on sec. NT
      for(c2=0; c2<MNT; des[c2] = 0.0, c2++) ;
      for(c2=0; c2<MNT; c2++)
         for(c3=0; c3<MNT; c3++)             //c2 destroying, c3 destroyed
            des[c3] += NtInp2[c][c2] * SynEq[c][c2] * CAT[c2][c3];
      for(c2=0; c2<MNT; c2++) //convert to surviving-NT factor
         if( (NtInp2[c][c2] <= 0.0) || (NtInp2[c][c2] < des[c2]) )
            des[c2] = 0.0;
            else des[c2] = (NtInp2[c][c2] - des[c2]) / NtInp2[c][c2];
      //destruction & decay of cell's total NTs (and AuxInp)
      AuxInp[c] *= HL_A;                   //auxiliary input decays
      for(c2=0; c2<MNT; c2++) {
         NtInp2[c][c2] *= des[c2];         //destroy total sec.NT
         NtInp1[c][c2] *= HL_P[c2];        //total pri decays
         NtInp2[c][c2] *= HL_S[c2];        //total sec decays
         }
      for(c2=0; c2<SynNum[c]; c2++) {
         syn_typ = SynType[c][c2];
         SyNt2[c][c2] *= des[syn_typ];     //destroy syn-sec-NT
         //synaptic growth al-juarismi
         if(ACTIVE[cell_typ][syn_typ]) {
            synAct = 1.0;
            for(c3=0; c3<MNT; c3++)
               //if(FOR[syn_typ][c3] > 0.0)//~
               if(c3 == syn_typ)
                  synAct *= SyNt2[c][c2] * FOR[syn_typ][c3];
                  else
                  synAct *= NtInp2[c][c3]*SynEq[c][c3]*FOR[syn_typ][c3];
            if( (synAct > 1.0) && (log(synAct) > SynWei[c][c2]) )
               SynWei[c][c2] = log(synAct);
            }
         SyNt1[c][c2] *= HL_P[syn_typ];                //syn pri decays
         SyNt2[c][c2] *= HL_S[syn_typ];                //syn sec decays
         }
      }//for all cells

   for(c=0; c<N_C; State[c] = next_state[c], c++) ; //update cell's State
}/*compute*/

/**************************** MAIN **********************************/

void main(void) {
/*
 CELL'S PARTS.  remember: no data structure can be > 64 K
 not declared together as "struct" for reasons of speed
 float is used to reduce memory requirements
*/
int          Type   [N_C],           //cell's default neurotransmiter
             State  [N_C],           //cell's "clock" (0 == depolarization)
             LastTir[N_C],           //last wavelength
             SynNum [N_C];           //number of synapses declared
float        AuxInp [N_C],           //"electrode" for external stimulation
             NtInp1 [N_C] [MNT],     //total store of primary NTs
             NtInp2 [N_C] [MNT],     //idem secondary
             SynEq  [N_C] [MNT];     //equalizes number of syn. by NT
int          SynType[N_C] [MAX_SYN], //types of declared synapses
             *Syn   [N_C] [MAX_SYN]; //address of neighbors (points to State)
    /*the rest is declared static because it won't fit in the stack*/
static float SynWei [N_C] [MAX_SYN], //weights of synapses
             SyNt1  [N_C] [MAX_SYN], //amount of primary NT
             SyNt2  [N_C] [MAX_SYN]; //amount of secondary NT
   long loops,c; /*main's variables*/

   cold_start(Type,SynNum,SynEq,SynType,Syn); /*clean memory up*/
   if (struc_file(Type,State,SynNum,SynType,Syn) ) return;//load struc01.bck
   if (params_file(1)) return;                            //load param01.bck
   reset(State,LastTir,AuxInp,NtInp1,NtInp2,SynWei,SyNt1,SyNt2);
   hl_factors();                 /*equalize for number of synapses??*/
   setEqu(SynNum,SynEq,SynType); //cargar desde struc_file??
   plot_World();                 //show frog's world
   show_LMRG();
   while (1) {
      loops = menu(Type,State,LastTir,SynNum,  //main menu, wait for command
                   AuxInp,
                   NtInp1,NtInp2,SynEq,
                   SynType,Syn,
                   SynWei,SyNt1,SyNt2);
      if(!loops) break; //exit bck on [esc]
      for(c=0; c<loops; c++) { //if loops > 0  /*main loop...*/
         Gen++;
         if(STI>0.0) retina(AuxInp); //if retina on, input world view
         compute(Type,State,LastTir,SynNum, //*compute metabolismi*
                 AuxInp,
                 NtInp1,NtInp2,SynEq,
                 SynType,Syn,
                 SynWei,SyNt1,SyNt2);
         if(STI>0.0) motor_navigation(LastTir,AuxInp,0); /*unused return*/
         show_LMRG();
         plot_waves(Type,State,NtInp1,NtInp2,SynEq); //show EEG
         plot_depo(State,LastTir); //show "firing" cells (wavelength->color)
         if( kbhit() ) { getch(); break; } //if a key was pressed, stop->menu
         }
      }                                        /*...ends here*/
   if(!TEST) closegraph(); //to use ALT F5 after program's end
}//main
/* This is the end, my only friend, the end. line 1770 05.10.95 05:08*/
